/****************************************************************
** File name:   ESP32_ADC.cpp
** Descrizione: Libreria per rilevare una grandezza a valle dell'ADC
**              
** E-mail: us.newsoft@gmail.com
** Autore: Salvatore Urciuolo
** Data:   07/06/2018 
****************************************************************/
#include "ESP32_ADC.h"

ESP32_ADC::ESP32_ADC(void)
{
}

ESP32_ADC::~ESP32_ADC(void)
{
}

/*
**   DESCRIZIONE  : Ritorna versione e data della libreria
**                  
**   PARAMETRI IN : N/A
**                 
**
**   RETURN       : 
**                  versione e data della libreria
**                  
*/
char* ESP32_ADC::getVersion()
{
	return versw;
}
/*
**   DESCRIZIONE  : Inizializzazione dell'istanza
**
**
**   PARAMETRI IN :
**                 [adc_width_res]  risoluzione in bit dell'ADC
**                 [atten]          Attenuazione da caratterizzare
**                 [channel_n]      numero del canale utilizzato
**
**   RETURN       : 
**                  RET_OK                       funzione terminata con successo
**                  ERR_CONFIG_ADC_RESOLUTION    errore nella fase configurazione della risoluzione dell'ADC
**                  ERR_CONFIG_CHANNEL           errore nella fase configurazione del canale
**
*/
int ESP32_ADC::init(int adc_width_res,int channel_n,int atten,int numSamples )
{
  int iEsito=RET_NOK;
    
  //configura ADC1 la risoluzione in bit  
  iEsito =  adc1_config_width((adc_bits_width_t)adc_width_res);
  if (iEsito == RET_OK)
  {
	//imposta il canale e la sua attenuazione
	iEsito = adc1_config_channel_atten((adc1_channel_t)channel_n,(adc_atten_t)atten);
	if (iEsito == RET_OK)
	{
	   // Calculate ADC characteristics i.e. gain and offset factors
	   esp_adc_cal_characterize((adc_unit_t)ADC_UNIT_1,(adc_atten_t)atten,(adc_bits_width_t)adc_width_res,DEFAULT_VREF,&m_characteristics);
	   
	   if ((numSamples <=0) || (numSamples > NUM_SAMPLES ))
	   {
			if (numSamples <= 0)
				m_numSamples = 1;
			else if (numSamples > NUM_SAMPLES_MAX)
				m_numSamples = NUM_SAMPLES_MAX;			
			else
				m_numSamples= NUM_SAMPLES;
	   }
	   else
		   m_numSamples = numSamples;
	}
  }
    
  return iEsito;
}

/*
**   DESCRIZIONE  : Legge il valore della tensionde dall'ADC espresso in Volt
**
**
**   PARAMETRI IN : N/A
**                 
**
**   RETURN       : 
**                  valore della tensione espressa in volt
**                  -1 in caso di errore
*/
float ESP32_ADC::readVoltage()
{   int iEsito=RET_NOK;
	uint32_t val = 0;
	uint32_t mV;
	float voltage=0;
	int countSamples=0;

	for(int i = 0; i < m_numSamples; i++) 
	{
		iEsito = esp_adc_cal_get_voltage((adc_channel_t)ADC1_CHANNEL_5,&m_characteristics,&mV);
		if (iEsito == RET_OK)
		{
		   val = val + mV;
		   countSamples++;
		}
		else
		{
			//ESP_ERR_TIMEOUT: Error, timed out attempting to read ADC
			//ESP_ERR_INVALID_ARG: Error due to invalid arguments
			countSamples--;
		}
		delay(1);
	}
  
	if (countSamples>0)
	{ 
	    val = val / countSamples;
		//si converte il valore espresso in mV --> volt
		voltage = val/ 1000.00;
	}
	else
	{
		voltage = -1;
	}
  return voltage;   
}

/*
**   DESCRIZIONE  : Ritorna il numero dei campioni utilizzati nella fase
**                  di subsampling
**   PARAMETRI IN : N/A
**                 
**
**   RETURN       : 
**                  numero dei campioni
**                  
*/
int ESP32_ADC::getNumSamples(void)
{
	return m_numSamples;
}
